clc; clear; close all

addpath fanChart

save_out = 1;  % set to 1 in order to save the results

unc = 0; % this is usually zero - set to 1 if you want to redraw the X's

T_list = [10;1000];   % sample sizes
n_Ts   = numel(T_list);

models_list = [1;2;3];          % selection of the models
n_models    = numel(models_list);

% generate N_outer distributions of the p-values, one for each draw of the X's
% Note: in CG 2019, N_outer is called M
N_outer = 1000;
% N = numeber of MC draws within each of the N_outer trials
N = 100000;   
% parameters for estimations of the asymptotic critical values
T_asy = 100000;    
N_asy = 100000;    


% intervals and steps for plotting the CDFs of p-values and lengths
steps = 1/200;
x_axis = (0:steps:1).'; % for CDF of pvalues
x_axis_l = (0:steps:3).'; % for CDF of lengths

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE UNCONDITIONAL CRITICAL VALUES for all models             %
% using the mixed normal representation of the asympt. distribution %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rng(11);            % Draw from the same seed each time

asy_cv = zeros(n_models,2);
res_asy = zeros(N_asy,1);
for i = 1:N_asy
    X = cumsum(randn(T_asy ,1))/sqrt(T_asy);
    res_asy(i,1) =randn(1,1)*sqrt(T_asy/(X.'*X));
end

res_asy = sortrows(abs(res_asy),1);
for model_index = 1:n_models
    model = models_list(model_index);
    asy_cv(model_index,1) = -quantile(res_asy,0.90);
    asy_cv(model_index,2) = -asy_cv(model_index,1);
    
    if model == 3
        asy_cv(model_index,:) = asy_cv(model_index,:)*sqrt(2/101);
    end
    
    fprintf('\n');
    fprintf('Model = %d\n', models_list(model_index));
    fprintf('Asy cv (lower) = %f\n', asy_cv(model_index,1));
    fprintf('Asy cv (upper) = %f\n', asy_cv(model_index,2));
    fprintf('Number of obs. for estimation = %f\n', T_asy);
    
end

tic;

rng(11);            % Draw from the same seed each time

for t_index = 1:size(T_list,1)
    
    T  = T_list(t_index);
    
    for model_index = 1:n_models
        
        model = models_list(model_index);

        savename = sprintf('T%d_Model%d.mat', T, model);
        
        res_full_CDF    = zeros(N_outer, size(x_axis,1));
        res_full_length = zeros(N_outer, size(x_axis_l,1));
        
        res_full  = zeros(N_outer, 3);
        % res_full(:,1) contains the average coverage (over N) of the boots CI
        % res_full(:,2) contains the average coverage (over N) of the asymptotic CI
        % res_full(:,3) contains the average length (over N) of the
        % bootstrap CI relative to the unconditional one
        
        for k = 1:N_outer % Outer cycle (in CG, N_outer is called "M")

            if mod(k,50) == 0
                 fprintf('T = %d, Model = %d, k = %d/%d\n', T, model, k, N_outer);
            end
            
            res = zeros(N,7);
            
            % Generate the X's
            if model == 1
                epsx = randn(T,1);
                X = cumsum(epsx(1:T,:));
            end
            if model == 2
                csi = randn(T,1);
                eta = zeros(T,1);
                eta(1) = csi(1)*sqrt(5/2);
                for mm = 2:T
                    eta(mm,:) = csi(mm,:).*sqrt(1+0.6*(eta(mm-1,:).^2));
                end
                X = cumsum(eta(1:T,:));
            end
            if model == 3
                ir = 9;
                mX = randn(T,1);
                mX = mX.*(1+ir*binornd(1,0.5,[T 1]));
                p=exp(-mX.*mX/2/(1+ir)^2)./(exp(-mX.*mX/2/(1+ir)^2)+(1+ir)*exp(-mX.*mX/2));
                X = cumsum(mX(1:T,:));
            end
            
            % inner cycle (over the N replications)
            for i = 1:N
                
                % generate the Y's
                if model == 1
                    y = randn(T,1);
                end
                if model ==  2
                    csi2 = randn(T,1);
                    eps = zeros(T,1);
                    eps(1)=csi2(1)*sqrt(5/2);
                    for mm = 2:T
                        eps(mm) = csi2(mm)*sqrt(1+0.3*(eps(mm-1)^2)+0.3*(eta(mm-1)^2));
                    end
                    y = eps;
                end
                if model == 3
                    mY=randn(T,1);
                    y=abs(mY).*(1-2*(rand(T,1) <= p(1:T,:)));
                end
                
                % estimation of beta and omega
                M = 1/(X.'*X);
                bhat = M*(X.'*y);
                omegahat = std(y-X*bhat);
                
                % [FULLY PARAMETRIC] bootstrap confidence set around beta hat
                ci_u = bhat+norminv(0.95)*(sqrt(M)*omegahat);
                ci_l = bhat-norminv(0.95)*(sqrt(M)*omegahat);
                
                % Unconditional confidence set around beta hat
                ci_u_unc = bhat+asy_cv(model_index,2)/T;
                ci_l_unc = bhat+asy_cv(model_index,1)/T;
                
                % Check coverage
                if (ci_u > 0)*(ci_l < 0) == 1
                    cov = 1;
                else
                    cov = 0;
                end
                if (ci_u_unc > 0)*(ci_l_unc < 0) == 1
                    cov_unc = 1;
                else
                    cov_unc = 0;
                end
                
                p_val = normcdf(bhat/(sqrt(M)*omegahat));
                res(i,1) = p_val;       % bootstrap pvalues
                res(i,2) = cov;         % 1 if CS covers beta
                res(i,3) = ci_l;        % lower bound of the bootstrap CS
                res(i,4) = ci_u;        % upper bound of the bootstrap CS
                res(i,5) = sqrt((T^2)*M*omegahat);
                res(i,6) = cov_unc;     % 1 if asy CS covers
                res(i,7) = ci_u_unc-ci_l_unc;   % length of asy CS (fixed)
                
            end % end of loop for inner cycle (N replications)
                       
            % Construct the ECDFs for the k-th outer cycle
            F = zeros(size(x_axis,1),1);
            for j = 1:size(x_axis,1)
                F(j) = mean(res(:,1) <= x_axis(j));
            end
            res_full_CDF(k,:) = F.';
            
            F = zeros(size(x_axis_l,1),1);
            for j = 1:size(x_axis_l,1)
                F(j) = mean( ((res(:,4)-res(:,3))./res(:,7)) <= x_axis_l(j));
            end
            res_full_length(k,:) = F.';
            
            res_full(k,1) = mean(res(:,2));     % mean coverage of bootstrap CS
            res_full(k,2) = mean(res(:,6));     % mean coverage of asympt. CS
            res_full(k,3) = mean((res(:,4)-res(:,3))./res(:,7)); % mean coverage
            
        end % end of loop for outer cycle k (N_outer replications)

        % -- Save results --
        if save_out == 1
            save(savename, 'x_axis', 'x_axis_l', 'res_full', 'res_full_CDF', 'res_full_length');
        end


        % ------------------
    end % end of the loop for Models
end % end of the loop for T

toc;


